<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use Carbon\Carbon;
use App\Models\Order;
use App\Models\Waste;
use App\Models\Container;
use App\Models\Location;

use Illuminate\Support\Facades\Mail;
use App\Mail\OrderEmail;

class OrderController extends Controller
{
    protected $perPage = 20;
    protected $minDays = 3;

    /**
     * All Orders
     */
    public function index() {

        $wastes = Waste::all();
        $locations = Location::all();
        $containers = Container::all();

        $orders = Order::paginate($this->perPage);

        return view('admin.orders.AllOrders',['orders' => $orders,'containers' => $containers, 'wastes' => $wastes, 'locations' => $locations]);
    }

    /**
     * Display the Edit View
     */
    public function edit($id) {
        $wastes = Waste::all();
        $locations = Location::all();
        $containers = Container::all();
        $order = order::findOrFail($id);
        $status = $this->orderStatus();

        if(!$order) {
            return redirect()->route('admin.orders')->with('error', 'Order not found.');
        }

        return view('admin.orders.EditOrder',['order' => $order, 'containers' => $containers, 'wastes' => $wastes, 'locations' => $locations, 'status' => $status]);
    }

    /**
     * Update Order
     */
    public function update(Request $request, $id) {

        $order = Order::findOrFail($id);

        if(!$order) {
            return redirect()->route('admin.orders')->with('error', 'Order not found.');
        }

        $request->validate([
            'status' => ['required', 'string'],
            'waste' => ['required', 'integer'],
            'bins' => ['required', 'integer'],
            'delivery' => ['required', 'string'],
            'collection' => ['required', 'string'],
            'name' => ['required', 'string'],
            'email' => ['required', 'string'],
            'phone' => ['required'],
            'address' => ['required', 'string'],
            'postcode' => ['required', 'string'],
            'price' => ['required'],
        ]);

        $price = $this->priceCalculation($request->postcode,$request->waste,$request->bins,$request->delivery,$request->collection);

        $order->update([
            'status' => $request->status,
            'waste' => $request->waste,
            'bins' => $request->bins,
            'delivery' => $request->delivery,
            'collection' => $request->collection,
            'name' => $request->name,
            'email' => $request->email,
            'phone' => $request->phone,
            'address' => $request->address,
            'suburb' => $request->suburb,
            'postcode' => $request->postcode,
            'instructions' => $request->instructions,
            'price' => $price,
        ]);

        return redirect()->back()->with('success', 'Order updated Successfully.');
    }

    /**
     * Order Status list
     */
    public function orderStatus() {
        $status = ['Completed','Pending','Processing'];
        return $status;
    }

    /**
     * Calculate Price
     */
    private function priceCalculation($postcode,$wasteType,$sizeContainer,$deliveryDate, $collectionDate) {

        $price = 0;
        $days = 0;

        if(!empty($deliveryDate) && !empty($collectionDate)) {
            $start = Carbon::parse($deliveryDate);
            $end = Carbon::parse($collectionDate);

            $days = $start->diffInDays($end);
        }

        if(!empty($postcode)) {
            $locationData = Location::where('zip',$postcode)->first();
            $locationPrice = $locationData['price'];
            if($locationPrice > 0) {
                $price += $locationPrice['price'];
            }
        }

        if(!empty($wasteType)) {
            $wasteData = Waste::where('id',$wasteType)->first();
            $basePriceDays = $wasteData['base_price_7days'];
            $basePriceDay = $wasteData['base_price_1day'];
            $containerPrice = unserialize($wasteData['container_prices']);
            $extraHireagePrice = $wasteData['extra_hireage_amount'];
            $extraHireageDays = $wasteData['extra_hireage_days'];
            $daterangePrices = unserialize($wasteData['daterange_prices']);

            $addContainerPrice = false;
            $datePriceCounter = 0;

            if($days <= $this->minDays) {
                $price += $basePriceDay;
            }
            else if($basePriceDays > 0) {
                $price += $basePriceDays;
            }

            if($days > $extraHireageDays) {
                $price += ($days - $extraHireageDays) * $extraHireagePrice;
            }

            if(!empty($daterangePrices)) {
                foreach($daterangePrices as $daterange) {
                    $dates = $daterange['dates'];

                    list($startRange, $endRange) = explode(' - ', $dates);

                    $isEqualStartRange = date('m/d/Y',strtotime($startRange));
                    $isEqualEndRange = date('m/d/Y',strtotime($endRange));

                    $startRange = Carbon::createFromFormat('m/d/Y', trim($startRange));
                    $endRange = Carbon::createFromFormat('m/d/Y', trim($endRange));

                    if(!empty($start)) {
                        $isWithinRange = $start->greaterThanOrEqualTo($startRange) && $start->lessThanOrEqualTo($endRange);
                        $isEqualStart = date('m/d/Y',strtotime($start));
                        $isEqualWithRange = ($isEqualStartRange == $isEqualStart) || ($isEqualStartRange == $isEqualEndRange);

                        if($isWithinRange || $isEqualWithRange) {
                            if(isset($daterange['sizes'])) {
                                $dateSizes = $daterange['sizes'];
                                if(in_array($sizeContainer,$dateSizes)) {
                                    if(isset($daterange['price'])) {
                                        $datePrice = $daterange['price'];
                                        $price += $datePrice;
                                    }
                                }
                                else {
                                    $datePriceCounter++;
                                }
                            }
                            else {
                                if(isset($daterange['price'])) {
                                    $datePrice = $daterange['price'];
                                    $price += $datePrice;
                                }
                            }
                            break;
                        }
                        else {
                            $isWithinRange = $start->gte($startRange) || $start->lte($endRange);
                        }
                    }
                }
            }
            else {
                $datePriceCounter++;
            }

            if($datePriceCounter > 0) {
                $addContainerPrice = true;
            }

            if($addContainerPrice && !empty($containerPrice)) {
                if(isset($containerPrice[$sizeContainer])) {
                    $price += $containerPrice[$sizeContainer];
                }
            }
        }

        return $price;
    }



    public function sendEmail()
    {
        $order_id = 10;

        $orderData = Order::where('id',$order_id)->first();
        $data = [];

        $data['id'] = $orderData->id;

        $wasteDetails = Waste::where('id',$orderData->waste)->first();
        $binDetails = Container::where('id',$orderData->bins)->first();
        $locationDetails = Location::where('zip',$orderData->postcode)->first();


        $data['waste'] = $wasteDetails->name;
        $data['bin'] = $binDetails->name;
        $data['delivery'] = date('F j, Y',strtotime($orderData->delivery));
        $data['collection'] = date('F j, Y',strtotime($orderData->collection));

        $data['postcode'] = $locationDetails->zip.' - '.$locationDetails->location;

        $data['name'] = $orderData->name;
        $data['email'] = $orderData->email;
        $data['phone'] = $orderData->phone;
        $data['address'] = $orderData->address;
        $data['suburb'] = $orderData->suburb;
        $data['instructions'] = $orderData->instructions;

        if($orderData->payment == 'delivery') {
            $paymentMethod = 'Pay on Delivery';
        }
        else if($orderData->payment == 'bank') {
            $paymentMethod = 'Pay by Bank';
        }

        $data['payment'] = $paymentMethod;
        $data['price'] = '$'.$orderData->price;

        Mail::to('calsmith348@gmail.com')->send(new OrderEmail($data));

        // print_r($data);

        return 'Email sent successfully!';
    }
}
